;
; Pulse generator panel interface
; Piotr Wjtowicz
; 03.2018
; mpasm (v5.76)
;
#define SEQUENCE
    
    radix		DEC	
    processor		16f505
    
 __CONFIG _MCLRE_OFF & _CP_OFF & _WDT_OFF & _IntRC_OSC_RB4EN

; The pinology
#define	    DS18B20_PIN	    RC4
#define	    DS18B20_PORT    PORTC
#define	    DS18B20_TRIS    PORTC
#define	    TMR_50MS_INIT   20			; definition of init value timer 50ms
#define	    COUNT tmp0

#define	    SW1_PORT	PORTB
#define	    SW1_PIN	RB3			; pin 4
#define	    SW2_PORT	PORTB
#define	    SW2_PIN	RB4			; pin 3
#define	    SW3_PORT	PORTB
#define	    SW3_PIN	RB1			; pin 12
 
#define	    LED1_PORT	PORTB
#define	    LED1_PIN	RB5			; pin 2
#define	    LED2_PORT	PORTC
#define	    LED2_PIN	RC3			; pin 7
#define	    LED3_PORT	PORTC
#define	    LED3_PIN	RC0			; pin 10
 
#define	    GEN_PORT	PORTC 
#define	    GEN_ENA	RC1			; pin 9 
#define	    HL_ENA	RC5			; pin 5
#define	    LH_ENA	RC2 			; pin 8
 
; General purpose regs: 08h..1Fh
BANK0			equ	    H'08'
main_state		equ	    H'08'	; state machine board
    errorlevel -207		
    PRESENCE_FLAG	equ 0	;bit 0 - PRESENCE_FLAG last device presence 0:present 1:absent
    MATCH_DSID_FLAG	equ 1	;bit 1 - MATCH_DSID_FLAG last device ID 0:fail 1:ok
    MATCH_CRC_FLAG	equ 2	;bit 2 - MATCH_CRC_FLAG last crc 0:nok 1:ok
    PULSE_H_ON		equ 3	;bit 3 - driving MOSFET by non-inverted pulse
    PULSE_L_ON		equ 4	;bit 4 - driving MOSFET by inverted pulse
    ALARM_ON		equ 5	;bit 5 - ALARM_ON alarm encountered 0:off 1:on
    RES6		equ 6	;bit 6 - reserved
    RES7		equ 7	;bit 7 - alarm over temp
    errorlevel +207
			
cblock	0x09
delay1_cnt		;equ	    H'09'	; register used in delay function			
delay2_cnt		;equ	    H'0A'	; register used in delay function
seq1_ptr		;equ	    H'0B'	; fan speed pulse
fsregt			;equ	    H'0C'	; fan speed rotation register
ds_txbyte_ptr		;equ	    H'0D'	; debug counter to simulation changing of temperature
tmp1			;equ	    H'0F'	; id read from DS ROM 
IOBYTE			;equ	    H'10'
tmp0			;equ	    H'11'	; Address 0x23
ONE			;equ	    H'12'	; Keep track of bits
TMSB			;equ	    H'13'	; Store the MSB
TLSB			;equ	    H'14'	; Store the LSB
temp			;equ	    H'15'	; temperature
COM_BUF			;equ	    H'16'	; COM data buffer
COM_PTR			;equ	    H'17'	; pointer to sending byte
cnt1			;equ	    H'18'
cnt2			;equ	    H'19'
ds_conv_del_cnt		;equ	    H'1A'
temp_real		;equ	    H'1B'
crc_reg			;equ	    H'1C'
crc_input		;equ	    H'1D'
crc_code		;equ	    H'1E'
delay3_cnt		;equ	    H'1F'	; tris register state
tris_masq_c		;equ	    H'20'
endc
		
BANK1			equ	    H'30'
cblock	0x30
endc
			
BANK2			equ	    H'50'
; ledState register description
    LS			equ	    0
    LBL			equ	    1
    LBE			equ	    2
    LBH			equ	    3
    LP			equ	    7

;struct of sw(x)state
switch_pressed		equ	    1
switch_deb		equ	    2
switch_pressed_2s	equ	    3
SWPN			equ	    7 	
#define			TIME_DEB    3
#define			TIME_1S	    50
cblock	0x50
sw1state
sw1cnt
sw2state
sw2cnt		
sw3state
sw3cnt	
swRegPtr	
ledBlinkCnt
ledState	
led1State
led2State
led3State
ledRegPtr
tmr_50ms_cnt
endc
	
; Constans 
optreg			equ	    b'11001111'	    ; wdt prescaler 128   
STX			equ	    2
ETX			equ	    3

#define LED_BLINK_DIV 5
#define LED_BLINK_LOW (95/LED_BLINK_DIV)
#define LED_BLINK_EQ (50/LED_BLINK_DIV)
#define LED_BLINK_HIGH (10/LED_BLINK_DIV)
#define LED_BLINK_CNT_INIT (100/LED_BLINK_DIV)
; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
RES_VECT	code    0x0000
    GOTO		init
PGM0		code	0x1			    ; let linker place main program      

; Includes ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
#include <p16f505.inc>				    ; processor specific variable definitions
#include "macros.inc"
#include "com.inc"


Seqs macro:    ; generates entrypoint to sub-sequences of code
    local s = NO_PROC_SEQ1
    while s > 0 
	GOTO	seq1_proc#v(s)	
s = s - 1
    endw
    endm
	
gpio_masq_b	equ	(1 << RB0) | (1 << RB1) | (1 << RB4)
gpio_masq_c	equ	0
#define TRIS_MASQ_C_INIT 0	

option_masq	equ	(1 << PS0) ; prescaler 1:4
#define	    NO_PROC_SEQ1	19

    
init: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    MOVLW	option_masq
    option
    
    MOVLW	gpio_masq_b
    TRIS 	PORTB			    ; set default pins configuration

    MOVLW	gpio_masq_c
    TRIS 	PORTC			    ; set default pins configuration

    ; Timer configuration
    LedOff	1
    LedOff	2
    LedOff	3

    CLRF	main_state
    BSF		main_state, MATCH_CRC_FLAG
    
    banksel	BANK1
    MOVLW	TMR_50MS_INIT/2
    MOVWF	tmr_50ms_cnt
    
    MOVLW	NO_PROC_SEQ1
    banksel	BANK1
    MOVWF	seq1_ptr
    
    ; led section
    banksel	BANK1
    CLRF	led1State
    CLRF	led2State
    CLRF	led3State
    
    ; switch section
    banksel	BANK2
    CLRF	sw1state
    CLRF	sw2state
    CLRF	sw3state
    MOVLW	LED_BLINK_CNT_INIT
    MOVWF	ledBlinkCnt
    CLRF	ledState
    CLRF	led1State
    CLRF	led2State
    CLRF	led3State 
    CLRF	ledRegPtr
    CLRF	swRegPtr
    CLRF	temp_real
    MOVLW	TRIS_MASQ_C_INIT
    MOVWF	tris_masq_c
    
    ; generator interface clearing
    BCF		GEN_PORT, GEN_ENA
    BCF		GEN_PORT, HL_ENA
    BCF		GEN_PORT, LH_ENA
main: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    ; hw test
    BSF		LED1_PORT, LED1_PIN
    BSF		LED2_PORT, LED2_PIN
    BSF		LED3_PORT, LED3_PIN
    
    DelayUs	1000000
    BCF		LED1_PORT, LED1_PIN
    BCF		LED2_PORT, LED2_PIN
    BCF		LED3_PORT, LED3_PIN
    
seq2: ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    banksel	BANK0
 
    ;reset timer
    CLRF	TMR0
    MOVLW	8
    MOVWF	TMR0
    ;wait until timer = 0
    
    incf	fsregt,F
    
print_data_ret:    
    
; handling temperature alarm 
#define		TEMP_AL_OFF	    (30*2)
#define		TEMP_AL_ON	    (32*2)
    
    banksel	BANK0
CheckAlarmOff:
    JG		temp_real, TEMP_AL_OFF, CheckAlarmOn	; if temp > tempAlOff GOTO   
    BCF		main_state, ALARM_ON			;  main_state.alarm = 0
CheckAlarmOn:
    JL		temp_real, TEMP_AL_ON, CheckAlarmEnd	; if temp < tempAlOn GOTO
    BSF		main_state, ALARM_ON			;   main_state.alarm = 1
CheckAlarmEnd:    

; handling of main_state states, check if is alarm
    banksel	BANK0
    BTFSS	main_state, ALARM_ON
    GOTO	a113					; if no alarm jump over
    BCF		main_state, PULSE_L_ON			; setting main_state
    BCF		main_state, PULSE_H_ON
    
    ; setting leds
    banksel	BANK2
    CLRF	led1State
    CLRF	led2State
    CLRF	led3State
    BSF		led3State, LBE

    ; turn off generator
    BCF		GEN_PORT, HL_ENA
    BCF		GEN_PORT, LH_ENA
    BCF		GEN_PORT, GEN_ENA
    
    GOTO	a6					; ommit switches handling and main_states 
a113:
    BTFSC	main_state, MATCH_CRC_FLAG		; check if ds type crc is valid
    GOTO	a114
a114:        
    ; switches keys mapping to states
    banksel	BANK2
    BTFSS	sw1state, switch_deb			; if switch1.switch_deb == 1
    GOTO	a1
    banksel	BANK0
    BSF		main_state, PULSE_L_ON			;  main_state.PULSE_L_ON = 1
    BCF		main_state, PULSE_H_ON			;  main_state.PULSE_H_ON = 0
a1:
    banksel	BANK2
    BTFSS	sw2state, switch_deb			; if switch2.switch_deb == 1
    GOTO	a2
    banksel	BANK0
    BSF		main_state, PULSE_H_ON		;  main_state.PULSE_H_ON = 1
    BCF		main_state, PULSE_L_ON		;  main_state.PULSE_L_ON = 0
a2:
    banksel	BANK2
    BTFSS	sw3state, switch_deb		; if switch3.switch_deb == 1
    GOTO	a3
    banksel	BANK0
    BCF		main_state, PULSE_H_ON		;  main_state.PULSE_H_ON = 0
    BCF		main_state, PULSE_L_ON		;  main_state.PULSE_L_ON = 0
a3:    
a112:    
    banksel	BANK0
    BTFSS	main_state, PULSE_H_ON		; if main_state.PULSE_H_ON == 1
    GOTO	a4
    ; set generator state
    BSF		GEN_PORT, LH_ENA
    BSF		GEN_PORT, GEN_ENA
    BCF		GEN_PORT, HL_ENA
    ; set leds state
    banksel	BANK2
    BSF		led2State, LBL
    CLRF	led1State
    CLRF	led3State
    GOTO	a6
a4:
    BTFSS	main_state, PULSE_L_ON		; else if main_state.PULSE_L_ON == 1
    GOTO	a5
    banksel	BANK2
    ; set generator state
    BSF		GEN_PORT, HL_ENA
    BSF		GEN_PORT, GEN_ENA
    BCF		GEN_PORT, LH_ENA    
    ; set led states
    BSF		led1State, LBH
    CLRF	led2State
    CLRF	led3State
    GOTO	a6
a5:    
    banksel	BANK2
    
    ; set generator state
    BCF		GEN_PORT, HL_ENA
    BCF		GEN_PORT, GEN_ENA
    BCF		GEN_PORT, LH_ENA
    
    BSF		led3State, LS
    CLRF	led1State
    CLRF	led2State
a6:    

seq2_finally:    
    MOVF	TMR0,W
    BTFSS	STATUS,Z
    GOTO	seq2_finally
    
    pagesel	seq1    
    GOTO	seq1    

    
PGM1		code	    0x200    


seq1:
    CLRF	TMR0
    MOVLW	8
    MOVWF	TMR0

    
    banksel	BANK0
    MOVF	seq1_ptr,W
    MOVWF	temp
#ifndef SEQUENCE
    GOTO	READ_TYPE_DS_LAB
READ_TYPE_DS_LAB_END:
#else
    DECF	temp,W
    ADDWF	PCL,F
    Seqs	NO_PROC_SEQ1        
#endif    
seq1_ret: 
    DECF	seq1_ptr,F
    BTFSC	STATUS,Z
    GOTO	seq1_init_ptr
    GOTO	seq1_next2
seq1_init_ptr:
    MOVLW	NO_PROC_SEQ1
    MOVWF	seq1_ptr
    
#include "ow_seq.inc"
seq1_next2:    
onlyLedHandle:    

    banksel	BANK2
    BCF		sw1state, SWPN
    BTFSC	SW1_PORT, SW1_PIN
    BSF		sw1state, SWPN

    BCF		sw2state, SWPN
    BTFSC	SW2_PORT, SW2_PIN
    BSF		sw2state, SWPN
    
    BCF		sw3state, SWPN
    BTFSC	SW3_PORT, SW3_PIN
    BSF		sw3state, SWPN    
    
    MOVLW	sw1state
    ADDWF	swRegPtr,W
    MOVWF	FSR
    
    BTFSS	INDF, SWPN			; if port(switch) == 1 //jezeli przycisk nie wcisniety
    GOTO	SwPressed	
    CLRF	INDF
    MOVLW	TIME_DEB			;  switch_cnt = TIME_DEB
    incf	FSR
    MOVWF	INDF
    DECF	FSR
    GOTO	SwEndProc
SwPressed:

SwCheckDebounced:
    BTFSS	INDF, switch_pressed		; else if switch_state.switch_pressed && switch_cnt == 0
    GOTO	SwCheckPressed1
    incf	FSR				; mowe ptr to cnt reg
    DECFsz	INDF,F
    GOTO	SwCheckPressed
    DECF	FSR				; return ptr to state reg
    BSF		INDF, switch_deb		;  switch_state.switch_deb = 1
    GOTO	SwEndProc
SwCheckPressed:
    DECF	FSR				; ret ptr to state reg
SwCheckPressed1:    
    BTFSC	INDF, SWPN			; else if port(switch) == 0 //jezeli przycisk wcisniety
    GOTO	SwEndProc
    BSF		INDF, switch_pressed		;  switch_state.switch_pressed = 1
SwEndProc:
    nop
    banksel	BANK2
    MOVF	swRegPtr,F
    BTFSS	STATUS,Z
    GOTO	swll3
    MOVLW	6
    MOVWF	swRegPtr
swll3:    
    DECF	swRegPtr
    DECF	swRegPtr

    banksel	BANK2
    MOVLW	led1State
    MOVWF	FSR
    MOVF	ledRegPtr,W
    ADDWF	FSR
    BTFSS	INDF, LS
    GOTO	sll2
    BSF		INDF, LP
    GOTO	sl2
sll2:    
    BTFSS	INDF, LBL
    GOTO	sl3
    JGE		ledBlinkCnt, LED_BLINK_LOW, sll1
    BCF		INDF, LP
    GOTO	sl2
sll1:    
    BSF		INDF, LP
    GOTO	sl2
sl3:
    BTFSS	INDF, LBE
    GOTO	sl4
    JGE		ledBlinkCnt, LED_BLINK_EQ, sle1
    BCF		INDF, LP
    GOTO	sl2
sle1:    
    BSF		INDF, LP
    GOTO	sl2
sl4:
    BTFSS	INDF, LBH
    GOTO	sl1
    JGE		ledBlinkCnt, LED_BLINK_HIGH, slh1
    BCF		INDF, LP
    GOTO	sl2
slh1:    
    BSF		INDF, LP
    GOTO	sl2
sl5:
sl1:
    BCF		INDF, LP
sl2:    
    ; set pointer to ledState register
    banksel	BANK2
    MOVF	ledRegPtr,F
    BTFSS	STATUS,Z
    GOTO	sll3
    MOVLW	3
    MOVWF	ledRegPtr
sll3:    
    DECF	ledRegPtr    
    
    ; set led pin state according to register
    BTFSS	led1State,LP
    BCF		LED1_PORT, LED1_PIN
    BTFSC	led1State,LP
    BSF		LED1_PORT, LED1_PIN
    
    BTFSS	led2State,LP
    BCF		LED2_PORT, LED2_PIN
    BTFSC	led2State,LP
    BSF		LED2_PORT, LED2_PIN
    
    BTFSS	led3State,LP
    BCF		LED3_PORT, LED3_PIN
    BTFSC	led3State,LP
    BSF		LED3_PORT, LED3_PIN    
    
seq1_nextProc1:
    DECFsz	tmr_50ms_cnt
    GOTO	skip_50ms_noreset
    ; handle 50s event
    MOVLW	TMR_50MS_INIT/2
    MOVWF	tmr_50ms_cnt
    DECFsz	ledBlinkCnt			; if --blinkCnt == 0
    GOTO	$+3				; set blinkCnt = LED_BLINK_CNT_INIT
    MOVLW	LED_BLINK_CNT_INIT
    MOVWF	ledBlinkCnt

    skip_50ms_noreset    

seq1_finally:
    MOVF	TMR0,W
    BTFSS	STATUS,Z
    GOTO	seq1_finally
    
    pagesel	seq2    
    GOTO	seq2

    END
